% Function to integrate a system of first-order 
% initial-value ODE's by the backward modified Euler method.
% 
% One Backward Euler step for the equation dy/dt = f(t,y) reads
% 
% y_n+1 = y_n + h * f(t_n+1, y_n+1)
% 
% The integrator accepts the same arguments as the built-in
% Matlab solvers ode23,.  The time step needs to be set
% by the options argument, field InitialStep.
% 
% Arguments:
%    rhsfun  = right-hand side function: returns f(t,y) 
%              as a column vector.
%    tspan   = time span (start time, usually zero, 
%              and stop time).
%    y0      = initial condition.
%    h       = step size; constant.
%
%
% Copyright (C) 2007, Petr Krysl
% 
function [ts, ys] = odeimeultrap (rhsfun, tspan, y0, options, varargin)
    h = options.InitialStep(1);
    if ~isfield( options,'RelTol')
        options.RelTol =1e-3;
    else
        if isempty(options.RelTol)
            options.RelTol =1e-3;
        end
    end
    nsteps = ceil ((tspan(2) - tspan(1)) / h);
    ts = zeros(nsteps+1,1);
    ys = zeros(nsteps+1,length(y0));
    t = tspan(1);
    ts(1) = t;
    ys(1,:) = y0';
    fac = [];
    options.threshold =1e-6+y0*1e-6;
    vectorized = 0;
    S = [];% generate full Jacobian matrix
    g=[];
    [dFdy,fac,g,nfevals,nfcalls] = numjac(rhsfun,t,y0,feval(rhsfun, t, y0, varargin{:}),options.threshold,fac,vectorized,S,g,varargin{:});
    options.drhsfundy =dFdy;
    for step = 2:nsteps+1
        if (ts(step-1) + h >tspan(2)), h=tspan(2)-ts(step-1); end
        ts(step) = ts(step-1) + h;
        ys(step,:) = Newton (rhsfun, h, ts(step-1), ys(step-1,:)', options, varargin{:});
    end
    return;
end

%
% Do one Backward Euler step for the equation dy/dt = f(t,y).
% 
% y_n+1 = y_n + h * f(t_n+1, y_n+1)
%
% Arguments: 
%   rhsfun = rhs function f(t,y)
%   h      = step length
%   tn     = time t_n
%   yn     = function value y_n
%   
function [yn1] = Newton (f, h, tn, yn, options, varargin)
    eta = options.eta;
    fn=feval (f,tn, yn, varargin{:});
    tp  = tn + h;
    yp  = yn + h * f(tp, yn, varargin{:});
    yn1 =yp;
    maximum_iteration = 20;
    J = eta*(h^2)/2 *options.drhsfundy*options.drhsfundy -(h/2) *options.drhsfundy +eye (length(yp));
    for iteration = 1: maximum_iteration
        f1 =f(tp, yn1, varargin{:});
        f0 =f(tn, (1-eta)*yn +eta*(yn1-h*f1), varargin{:});
        nyn1 =yn1 - J\(yn1 - yn - (h/2) * f1 - (h/2) * f0);
        yn1increment=nyn1-yn1; 
        if norm(yn1increment,inf)<options.RelTol*norm(max(max(abs(yn1)), options.threshold),inf)
            yn1 =nyn1; return;
        end
        yn1 =nyn1;
    end
    yn1increment(isnan(yn1increment)) = realmax;
    warning(['odeimeul: Not converged with ' num2str(maximum_iteration) ' iterations; Max increment ' num2str(norm(max(max(abs(yn1increment)), options.threshold),inf))]);
end

% function [yn1] = IMEuler_fminsearch (f, h, tn, yn, varargin)
%     tp  = tn + h;
%     yp  = yn + h * f(tp, yn, varargin{:});
%     function v =imeulerf(yn1, f, h, tp, yn, varargin)
%         f1 =f(tp, yn1, varargin{:});
%         v = yn1 - yn - h/2 * (f(tn, yn1-h*f1, varargin{:}) + f1);
%         v = v'*v ;
%     end
%     options =optimset('TolFun',((yn+yp)'*(yn+yp))/1e16,'MaxIter',1000); % ,'Display','iter','TolX',0
%     [yn1,fval] = fminsearch(@imeulerf, yp, options, f, h, tp, yn, varargin{:}); % optimset('disp','iter')
% end

