% Compute the material stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u1, u)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness (self, geom, u1, u)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    matstates = self.matstates;
    Xs = gather(geom, (1:get(geom,'nfens')), 'values', 'noreshape'); % reference coordinates
    Us = gather(u, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n
    U1s = gather(u1, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n+1
    for i=1:size(conns,1)
        conn =conns(i,:); % connectivity
        X=Xs(conn,:);           
        U=Us(conn,:);           
        U1=U1s(conn,:);           
        x = X + U; % previous known coordinates
        x1 = X + U1; % current coordinates
        for j=1:npts_per_gcell       % Loop over all integration points
            c =Ns{j}'*X;% physical location of the quadrature point
            J = x1' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            if (isempty(Rm))
                Nderx1 = Nders{j}/J;% derivatives wrt global coor
                NderX=Nders{j}/(X' * Nders{j});
            else
                Nderx1 = Nders{j}/(Rm'*J);% derivatives wrt local coor
                NderX=Nders{j}/(Rm'*X' * Nders{j});
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x1);
            if (Jac<=0),error('Non-positive Jacobian');end
            F1 = x1'*NderX;%def_grad(gcells(i), NderX, x1);
            F = x'*NderX;%def_grad(gcells(i), NderX, x);
            B = self.hBlmat(self,Ns{j},Nderx1,c,Rm);% strain-displacement
            D = tangent_moduli (mat, struct ('ms',matstates{i,j},'F',F1,'stiff_type','Eulerian'));
            Ke{i} = Ke{i} + (B'*(D*(Jac*w(j)))*B);
       end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
