%  Draw graphic representation for material states in all gcells.
%
% function draw_integration_points(self, gv, context)
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with typically mandatory fields
%    x=reference geometry field
%    u=displacement field to be used for the calculation of the stresses
%    u_displ=displacement field to be used for display
%    dT=temperature difference field
% with optional fields
%    scale = scale the representation of the material state graphic by this
%       factor
%    dT=temperature difference (with respect to the reference temperature);
%       temperature differences assumed to be zero if it is not supplied.
% Any other fields in the context struct may be interpreted by the
% implementation of the material: see the draw() function for the
% particular material.
%
function draw_integration_points(self, gv, context)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(context.x,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts'); % number of integration point
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    context.update_context= [];
    context.ms = [];
    xs =gather(context.x,(1:get(context.x,'nfens')),'values','noreshape');
    for i=1:ngcells
        conn =conns(i,:);
        X = xs(conn,:);% reference coordinates
        U_displ = gather(context.u_displ, conn, 'values', 'noreshape'); % displacement
        Uv = gather(context.u, conn, 'values'); % displacement
        if isfield(context,'dT')
            dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
        else
            dTs = zeros(nfens,1);
        end
        % Loop over all integration points
        for j=1:npts_per_gcell
            c =Ns{j}'*X;
            J = X' * Nders{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, X);
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = self.hBlmat(self,Ns{j},Ndersp,c,Rm);% strain-displacement
            context.D = tangent_moduli(mat,struct('xyz',c));
            context.update_context.strain = B*Uv;
            context.update_context.dT =Ns{j}'*dTs;
            context.xyz =Ns{j}'*(X+U_displ);
            context.Rm=Rm;
            draw(mat, gv, context);
        end
    end
    return;
end
