% Compute the current length of the element, the current element frame, 
% and the natural deformations  
%
% function [Lt,Ft,dN]=local_frames(x0,x1x2_vector,xt,RI,RJ)
% 
% Inputs:
% x0= array of node coordinates, one node per row, in initial configuration
% x1x2_vector= vector that lies in the x1-x2 local element coordinate
%      plane, in initial configuration
% xt= array of node coordinates, one node per row, in initial configuration
% RI,RJ=nodal rotation (orthogonal) matrix 
% 
% Outputs:
% Lt= current length of the element,
% Ft= current element frame (orthogonal rotation matrix) whose columns are unit
%      vectors: they are centered halfway between the current locations of
%      the nodes, vector 1 points from node I to node J, vector 3 is
%      orthogonal to the sum of the nodal cross-section frame vectors 2
% dN= vector of natural deformations; dN(1)= total change in length 
%      between configurations 0 and t; dN(2)= symmetric bending;
%      dN(3)= anti-symmetric bending;    dN(4)= symmetric bending
%      dN(5)= anti-symmetric bending; dN(6)=total axial torsion angle.
% 
function [Lt,Ft,dN]=local_frames(x0,x1x2_vector,xt,RI,RJ)
    % This is the element frame in the configuration t=0
    F0=zeros(3);
    F0(:,1)= (x0(2,:)-x0(1,:))'; 
    L0 =norm(F0(:,1));
    F0(:,1)=F0(:,1)/L0;
    F0(:,3)=cross(F0(:,1),x1x2_vector); F0(:,3)=F0(:,3)/norm(F0(:,3));
    F0(:,2)=cross(F0(:,3),F0(:,1));
    
    % The nodal cross-section frames are rotated by the nodal rotation matrices
    FtI=RI*F0;
    FtJ=RJ*F0;
    
    % Compute the element frame in configuration t
    Ft=zeros(3);
    Ft(:,1)= (xt(2,:)-xt(1,:))'; 
    Lt =norm(Ft(:,1));
    Ft(:,1)=Ft(:,1)/Lt;
    x1x2_vectort=FtI(:,2)+FtJ(:,2);
    Ft(:,3)=cross(Ft(:,1),x1x2_vectort); Ft(:,3)=Ft(:,3)/norm(Ft(:,3));
    Ft(:,2)=cross(Ft(:,3),Ft(:,1));
    
    % Components of FtI,FtJ in the element frame
    LFtI=Ft'*FtI;
    LFtJ=Ft'*FtJ;
    
    % C  COMPUTE NET DEFORMATIONS- DELTA L,TH1IJ,TH2I,TH2J,TH3I,TH3J
    % C  CHANGE IN LENGTH
    dN=zeros(6,1);
    % Total change in length between configurations 0 and t
    dN(1)= Lt-L0;
    % Total axial TORSION ANGLE
    dN(6)= (LFtJ(3,2)/LFtJ(2,2)-LFtI(3,2)/LFtI(2,2) -LFtJ(2,3)/LFtJ(3,3)+LFtI(2,3)/LFtI(3,3))/2;
    % Total rotation  angles of nodal cross-sections relative to the element frame
    TH2I=-LFtI(3,1)/LFtI(1,1);
    TH2J=-LFtJ(3,1)/LFtJ(1,1);
    TH3I= LFtI(2,1)/LFtI(1,1);
    TH3J= LFtJ(2,1)/LFtJ(1,1);
    dN(2)= TH3I-TH3J; % symmetric bending
    dN(3)= TH3I+TH3J; % anti-symmetric bending
    dN(4)= -TH2I+TH2J; % symmetric bending
    dN(5)= -TH2I-TH2J; % anti-symmetric bending
end
% C  PROCEDURE COMPUTES TRIADS: NODE TRIADS,CROSS-SECTION TRIADS ,
% C  ELEMENT TRIAD AND COORDINATES OF FICTITIOUS NODES IN THE CASE OF
% C  BEAM ELEMENTS ECCENTRICALLY CONNECTED TO NODES NI,NJ
% C  AND NET DEFORMATIONS FOR BEAM AND TRUSS ELEMENTS
% C  REQNUM=1 =>COMPUTE TRIADS AND FXTNI,...FZTNJ  ONLY
% C                           ************
% 
%       INTEGER REQNUM,I,J,K
%       DOUBLE PRECISION XNI,XNJ,XNK,YNI,YNJ,YNK,ZNI,ZNJ,ZNK,XTNI,XTNJ,
%      *   YTNI,YTNJ,ZTNI,ZTNJ,L0,LT,W(3),
%      *   B1I(3),B2I(3),B1J(3),B2J(3),ECC(4),
%      *   FXTNI,FXTNJ,FYTNI,FYTNJ,FZTNI,FZTNJ,DX,DY,DZ,
%      *   F0(3,3),VTI(3,3),VTJ(3,3),BTI(3,3),BTJ(3,3),FT(3,3),
%      *   NETDEF(6),TRFT(3,3),LVTI(3,3),LVTJ(3,3),EC1,EC2,EC3,EC4
% C.......................................................................
% 
% C  COMPUTE LOCAL TRIAD IN CONFIGURATION 0-TERMS IN GLOBAL COORDS
%       DX = XNJ-XNI
%       DY = YNJ-YNI
%       DZ = ZNJ-ZNI
%       F0(1,1)= DX
%       F0(2,1)= DY
%       F0(3,1)= DZ
%       L0 = DSQRT(DX*DX+DY*DY+DZ*DZ)
%       IF (L0 .LE. 0.0) THEN
% 	 WRITE(*,200)
% 	 STOP
%       END IF
%       W(1)= XNK-XNI
%       W(2)= YNK-YNI
%       W(3)= ZNK-ZNI
%       CALL MULTIC(F0(1,3),F0(1,1),W,3)
%       CALL MULTIC(F0(1,2),F0(1,3),F0(1,1),3)
%       CALL TRNORM(F0,3)
% C.......................................................................
% 
% C  COMPLETE NODE TRIADS BTI,BTJ-TERMS IN GLOBAL COORDS
%       DO 138,K=1,3
%          BTI(K,1)=B1I(K)
%          BTI(K,2)=B2I(K)
%          BTJ(K,1)=B1J(K)
%          BTJ(K,2)=B2J(K)
% 138   CONTINUE
%       CALL MULTIC(BTI(1,3),BTI(1,1),BTI(1,2),3)
%       CALL MULTIC(BTJ(1,3),BTJ(1,1),BTJ(1,2),3)
% 
% C  COMPUTE VTI,VTJ-TERMS IN GLOBAL COORDS
%       CALL MULTIM(BTI,F0,VTI,3,3,3)
%       CALL MULTIM(BTJ,F0,VTJ,3,3,3)
% 
% C  COMPUTE COORDINATES OF FICTITIOUS NODES
%       EC1=ECC(1)
%       EC2=ECC(2)
%       EC3=ECC(3)
%       EC4=ECC(4)
%       FXTNI=XTNI+VTI(1,1)*EC1+VTI(1,2)*EC2+VTI(1,3)*EC3
%       FYTNI=YTNI+VTI(2,1)*EC1+VTI(2,2)*EC2+VTI(2,3)*EC3
%       FZTNI=ZTNI+VTI(3,1)*EC1+VTI(3,2)*EC2+VTI(3,3)*EC3
%       FXTNJ=XTNJ+VTI(1,1)*EC4+VTI(1,2)*EC2+VTI(1,3)*EC3
%       FYTNJ=YTNJ+VTI(2,1)*EC4+VTI(2,2)*EC2+VTI(2,3)*EC3
%       FZTNJ=ZTNJ+VTI(3,1)*EC4+VTI(3,2)*EC2+VTI(3,3)*EC3
% C.......................................................................
% 
% C  COMPUTE ELEMENT TRIAD FT-TERMS IN GLOBAL COORDS
%       DX = FXTNJ-FXTNI
%       DY = FYTNJ-FYTNI
%       DZ = FZTNJ-FZTNI
%       LT = DSQRT(DX*DX+DY*DY+DZ*DZ)
%       IF (LT .LE. 0.0) THEN
% 	 WRITE(*,200)
% 	 STOP
%       END IF
%       FT(1,1)= DX
%       FT(2,1)= DY
%       FT(3,1)= DZ
%       W(1)= VTI(1,2)+VTJ(1,2)
%       W(2)= VTI(2,2)+VTJ(2,2)
%       W(3)= VTI(3,2)+VTJ(3,2)
%       CALL MULTIC(FT(1,3),FT(1,1),W,3)
%       CALL MULTIC(FT(1,2),FT(1,3),FT(1,1),3)
%       CALL TRNORM(FT,3)
% C  MATRIX FT IS TRANSF.MATRIX DIAGONAL SUBMATRIX
% 
%       IF(REQNUM.EQ.1)RETURN
% C.......................................................................
% 
% C  COMPUTE TERMS OF VTI,VTJ IN ELEMENT COORDS
%       DO 10 I=1,3
%       DO 10 J=1,3
%       TRFT(I,J)=FT(J,I)
%    10 CONTINUE
%       CALL MULTIM(TRFT,VTI,LVTI,3,3,3)
%       CALL MULTIM(TRFT,VTJ,LVTJ,3,3,3)
% C.......................................................................
% 
% C  COMPUTE NET DEFORMATIONS- DELTA L,TH1IJ,TH2I,TH2J,TH3I,TH3J
% C  CHANGE IN AXIS' LENGTH
%       NETDEF(1)= LT-L0
% C  TORSION ANGLE
%       NETDEF(2)= (LVTJ(3,2)/LVTJ(2,2)-LVTI(3,2)/LVTI(2,2)
%      *           -LVTJ(2,3)/LVTJ(3,3)+LVTI(2,3)/LVTI(3,3))/2.
% C  BENDING
%       NETDEF(3)=-LVTI(3,1)/LVTI(1,1)
%       NETDEF(4)=-LVTJ(3,1)/LVTI(1,1)
%       NETDEF(5)= LVTI(2,1)/LVTI(1,1)
%       NETDEF(6)= LVTJ(2,1)/LVTJ(1,1)
% 
%       RETURN
% 200   FORMAT(///'(''ERROR: TRIABT, BEAM LENGTH .LE. 0.0'')',//)
% C************************   END OF PROCEDURE   *************************
%       END
% #endif