% Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% 
% function [self,evs] = restoring_force(self, geom, dgeom1, Rfield1, dchi)
% 
% Return an elevecset object so the element vectors may be assembled.
%     
%     geom=initial geometry field; locations of nodes
%     dgeom1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%
% The element forces are computed in the local coordinate system and
% then transformed into the global coordinate system. 
% Equation numbers are taken from the field dchi. 
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
% Call as
% evs = restoring_force(feb, geom0, u1, Rfield1, dchi), or
% [feb,evs] = restoring_force(feb, geom0, u1, Rfield1, dchi)
% 
function [self,evs] = restoring_force(self, geom0, u1, Rfield1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(dchi,'dim');
    conns = get(gcells, 'conn'); % connectivity
    % Pre-allocate 
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dchi,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom0,'dim')*nfens,1);
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    Rs = gather(Rfield1, (1:get(geom0,'nfens')), 'values', 'noreshape'); % displacement in step n
    X1s=Xs+U1s;
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I1=get(gcells,'I1');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);% current directors at node 1
        RJ =reshape(Rs(conn(2),:),3,3);% current directors at node 2
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        P_N = natural_forces (self, E, G, A(i), I2(i), I3(i), J(i), L1, d_N);
        P = local_nodal_forces (self, P_N, L1);
        Fe{i} = -Te*P;% transform to global coordinates
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
