% Compute the stiffness matrices of the individual gcells.
%
% function ems = stiffness (self, geom, u)
%
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = stiffness (self, geom, u)
    gcells = get(self,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    % - volumetric terms
    pc_v = get(integration_rule, 'param_coords',1);
    w_v  = get(integration_rule, 'weights',1);
    n_v = get(integration_rule, 'npts',1);
    for j=1:n_v
        Ns_v{j} = bfun(gcells,pc_v(j,:));
        Nders_v{j} = bfundpar(gcells,pc_v(j,:));
    end
    % - shear terms
    pc_s = get(integration_rule, 'param_coords',2);
    w_s  = get(integration_rule, 'weights',2);
    n_s = get(integration_rule, 'npts',2);
    for j=1:n_s
        Ns_s{j} = bfun(gcells,pc_s(j,:));
        Nders_s{j} = bfundpar(gcells,pc_s(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self, 'mater');
    % Loop over all the geometry cells
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =get(geom,'values');
    Blmat = get(self,'Blmat');% strain-displacement matrix function handle
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:n_v
            c =Ns_v{j}'*x;% physical location
            J = x' * Nders_v{j};
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, Ns_v{j}, J, x);
            if (isempty(Rm))
                Ndersp = Nders_v{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders_v{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = Blmat(self,Ns_v{j},Ndersp,c,Rm);% strain-displacement
            D = tangent_moduli (mat,struct('kind',self.split));
            Ke{i} = Ke{i} + (B'*(D*(Jac*w_v(j)))*B);
        end
    end
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:n_s
            c =Ns_s{j}'*x;% physical location
            J = x' * Nders_s{j};
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, Ns_s{j}, J, x);
            if (isempty(Rm))
                Ndersp = Nders_s{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders_s{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = Blmat(self,Ns_s{j},Ndersp,c,Rm);% strain-displacement
            D = tangent_moduli (mat, ...
                struct('kind',[self.split '_shear']));
            Ke{i} = Ke{i} + (B'*(D*(Jac*w_s(j)))*B);
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
    
end
