% Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% 
% function [self,evs] = restoring_force(self, geom, dgeom1, Rfield1, dchi)
% 
% Return an elevecset object so the element vectors may be assembled.
%     
%     geom=initial geometry field; locations of nodes
%     dgeom1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%
% The element forces are computed in the local coordinate system and
% then transformed into the global coordinate system. 
% Equation numbers are taken from the field dchi. 
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
% Call as
% evs = restoring_force(feb, geom0, u1, Rfield1, dchi), or
% [feb,evs] = restoring_force(feb, geom0, u1, Rfield1, dchi)
% 
function [self,evs] = restoring_force(self, geom0, u1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(dchi,'dim');
    conns = get(gcells, 'conn'); % connectivity
    % Pre-allocate 
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        e1=gather(dchi,conns(i,1),'eqnums');
        eqnums{i} =[e1(1:3)];
        Fe{i} =[];
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    X1s=Xs+U1s;
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        dx=(x1- x0)-dot(self.direction,(x1- x0))*self.direction;
        distance=norm(dx);
        F1=zeros(3,1);
        if (distance > 0 )
            N=dx/distance;
            Fmag=self.force_function(conn,distance);
            F1=N*Fmag;% In global coordinates
        end
        Fe{i} = -F1;
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
