% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    tol=1000*eps;
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    if (isempty(self.T))
        self.T = Rn1'*feval(torque, t, Rn1);
    end
    Tn1=self.T;
    for i=1:nsteps
        % half-step with algorithm 1
        Omegamid=approx_newton(I, dt/2, I*Omegan1+dt/2*Tn1, tol);
        Rn=reortho(Rn1*rotmat(skewmat(dt/2*Omegamid)));
        Omegan = I\(Rn'*Rn1*(I*Omegan1+dt/2*Tn1));
        % followed by half-step with algorithm 2
        Rn1=Rn;
        Omegan1=Omegan;
        Omegamid=approx_newton(I, dt/2, I*Omegan1, tol);
        Rn=reortho(Rn1*rotmat(skewmat(dt/2*Omegamid)));
        Tn=Rn'*feval(torque, t+dt, Rn);
        Omegan = I\(Rn'*Rn1*I*Omegan1+dt/2*Tn);
        Rn1=Rn;    Omegan1=Omegan;    Tn1=Tn;    t=t+dt;
    end
    % swap variables for next step
    self.R = Rn;
    self.Omega = Omegan;
    self.T = Tn;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end

function Omegamid=approx_newton(I, dt, Pin1, tol)
    Kk=eye(3,3) - inv(I)*(dt/2*skewmat(Pin1));
    Omegamid=I\Pin1;
    i=0; maxi=50;
    res = Omegamid - I\(rotmat(skewmat(-dt/2*Omegamid))*Pin1);
    invIdt2 = inv(I) * dt^2/8;
    while (max(abs(res)) > tol)
        K = Kk + invIdt2 * (skewmat(Omegamid)*skewmat(Pin1) + skewmat(skewmat(Omegamid)*Pin1));
        Omegamid = Omegamid - K\res;
        res = Omegamid - I\(rotmat(skewmat(-dt/2*Omegamid))*Pin1);
        i=i+1;
        if (i > maxi)
            if (max(abs(res)) > 1e-9)
                warning(['Failed to converge: ||residual||=' num2str(norm(res))]);
            end
            break;
        end
    end
    return;
end