% Constructor of a square sparse system matrix. This class provides methods for
% constructing a system matrix by storing its lower triangle and the diagonal, 
% in-place factorization into
% L*D*L^T, and solution via forward and backward substitution.
%
% function retobj = sparse_ldlt_sysmat ()
%
% Parameters: none
% 
% Notes:
% -- This class does not support multiple assemble() calls. For instance, 
% it is not possible to do
%           A= assemble(A,ems1);
%           A= assemble(A,ems2);
% Rather, the above must be accomplished by concatenating the element 
% matrix objects and calling assemble() only once
%           A= assemble(A,cat(2,ems1,ems2));
% 
% -- This class optimizes multiple solutions with different right hand sides
% as it computes the Cholesky factorization of the system matrix at the
% time of the first solve, and then just does the forward and backward
% substitution for subsequent solves. Note that the factorization is the
% stored in-place (it overwrites the original matrix).
% 
% -- Only symmetric matrices are supported.  The matrix lower left
% triangle is stored under the assumption that the matrix is indeed symmetric. 
% 
% -- The forward/backward substitution can take advantage of the
% memory-conserving solvers of the package CXSparse, provided this package
% is installed.
% 
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>.
%
function self = sparse_ldlt_sysmat (varargin)
    class_name='sparse_ldlt_sysmat';
    parent=sysmat;
    initialized = false;
    Factorized =false;
    invalid_eqnum = get(sysmat, 'invalid_eqnum');
    A = [];
    D = [];
    neqns= 0;

    function assemble_method (ems)
        ne =length(ems);
        sl=0;
        for m = 1:ne
            eqnums=get(ems(m), 'eqnums');
            for j = 1:length(eqnums)
                sl =sl+length(eqnums{j})+round(1/2*length(eqnums{j})^2)+1;
            end
        end
        sppatti= zeros (1,sl);
        sppattj= zeros (1,sl);
        Ainit= zeros (1,sl);
        triplet =1;
        for m = 1:ne
            eqnums=get(ems(m), 'eqnums');
            mat    = get(ems(m), 'mat');
            if placesymm(ems(m))
                for j = 1:length(eqnums)
                    emask=eqnums{j}~=invalid_eqnum;
                    conns =eqnums{j} (emask);
                    amat =(mat{j}(emask,emask)+mat{j}(emask,emask)')/2;
                    for c= 1:length(conns)
                        for r= 1:length(conns)
                            gr =conns(r); gc=conns(c);
                            if (gr>=gc)
                                sppatti(triplet) =gr;
                                sppattj(triplet) =gc;
                                Ainit(triplet) =amat(r,c);
                                triplet =triplet +1;
                            end
                        end
                    end
                end
            else
                error ('Not implemented');
            end
        end
        A=sparse(sppatti(1:triplet-1), sppattj(1:triplet-1), Ainit(1:triplet-1), neqns, neqns);
    
        return;
    end
    self.assemble_method=@assemble_method;

    function x=solve_method (V)
        if (~Factorized)
            [A,D,p] = ldl(A,'vector');% A is now psychologically lower triangular
            Factorized = true;
        end
        x=0*V;
        x(p) = (A)'\(D\(A\V(p)));
    end
    self.solve_method=@solve_method;

    function retobj=start_method (dim)
        neqns =dim;
        initialized = false;
        Factorized = ~true;
        retobj =self;
    end
    self.start_method=@start_method;

    function val = get_method(varargin)
        if (nargin == 0)
            val = {{{'dim'}, {'dimension, array '}};
                {{'mat'}, {'sparse data matrix, array [dim,dim]'}};
                {{'mat_diag'}, {'diagonal sparse data matrix, array [dim,dim]'}};
                };
            return;
        else
            prop_name=varargin{1};
            switch prop_name
                case 'mat'
                    if ( Factorized )
                        val = A;
                    else
                        val = A+A'-spdiags(spdiags(A,0),0,size(A,1),size(A,2));
                    end
                case 'mat_diag'
                    val = D;
                case 'dim'
                    val = neqns;
                otherwise
                    error(['Unknown property name ''' prop_name '''!']);
            end
        end
        return;
    end
    self.get_method=@get_method;

    function retval=set_method(varargin)
        if (nargin == 1)
            retval = {};
            return;
        end
        prop_name=varargin{1};
        val=varargin{2};
        switch prop_name
            otherwise
                error(['Unknown property ''' prop_name '''!'])
        end
        retval=self; % return self
    end
    self.set_method=@set_method;

    self = class(self, class_name);
end
