%%  Cantilever beam, Euler buckling. Corotational beam
% 
% Classical Euler buckling of a cantilevered column.

% Parameters:
E=100;
nu=0.3;
L= 28; % Length of the beam
b = 1; % width
h = 3; % height
scale=5;
% For this load the first buckling parameter is 1.0. The next three are 9.0, 9.0, 25.0.
magn_cr = E*min([b,h])^3*max([b,h])/12*pi^2/(2*L)^2;

neigvs = 4;%number of eigenvalues


%% 
% Mesh
n=8;
tolerance=L/n/100;
Parameters=struct ('A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'J',b^3*h/3,...
    'x1x2_vector',[0 1 0], 'label',1);
[fens,gcells] = Beam_member([0,0,0;0,0,L],n,@gcellset_beam3,Parameters);


% The clamped end is
clampedn=fenode_select(fens, struct('box', update_box([],[0,0,0]), 'inflate', tolerance));
% The force is applied at the tip
tipn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));

%% 
% Properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));

%% 
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));

%% 
% Initialize configuration variables
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
u=0*clone(geom,'transl');
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));
rot=0*clone(geom,'rotat');
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

%% 
% Apply EBC's
ebc_fenids=[clampedn];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);


%% 
% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');


%% 
% Assemble and solve the discrete system for static displacement
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, u, Rfield, dchi));
% Load
F = sysvec;
F = start (F, neqns);
evs = loads(nodal_load(struct ('id',tipn,'dir',[3],'magn',[-magn_cr])), dchi);
F = assemble (F, evs);
% Solve 
dchi = scatter_sysvec(dchi, K\F);
load_displacement=somel(gather(dchi,tipn,'values','noreshape'),1:3)

u=slice(dchi,(1:3),'u');
rot=slice(dchi,(4:6),'rot');


%% 
% Display the deformed structure
gv=graphic_viewer;
gv=reset (gv,[]);
scale=200;
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));


%% 
% Set up the geometric stiffness
scale_down=1/10;
Kg = start (sparse_sysmat, neqns);
Kg = assemble (Kg, stiffness_geo(feb, geom, scale_down*u, update_rotation_field(Rfield,scale_down*rot), dchi));


%% 
% Solve the buckling problem
options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(-(get(Kg,'mat')+get(Kg,'mat')')/2/scale_down,(get(K,'mat')+get(K,'mat')')/2,...
    neigvs,'lm', options);
Omegas=(1./(diag(Omega)));

%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
end
disp(' ')


%% 
% Graphic display of the buckling modes
gv=graphic_viewer;
gv=reset (gv,[]);
scale=0.5;
w=clone(dchi,'w'); % make a copy of u
for i=1:neigvs
    ur = scatter_sysvec(w, W(:,i));
    u=slice(ur,(1:3),'u');
    rot=slice(ur,(4:6),'rot');
    wv = get(u,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    gv=reset (gv,[]);
    draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot,'colorfield',colorfield));
    draw_axes(gv, struct([]));
    title(['  Eigenvector ' num2str(i) ' eigenvalue ' num2str(Omegas(i)) ]);
    pause(2); % next eigenvector
end


