%%  lzzuddin, Elnashai Dome frame. Nonlinear statics.
% 
% Framed dome structure, loaded by concentrated force at the crown.
% Nonlinear static analysis, compared with results from the literature.

%%
%
% Reference: EULERIAN FORMULATION FOR LARGE-DISPLACEMENT ANALYSIS OF SPACE
% FRAMES, by B.  A.  lzzuddin I and A.  S.  Elnashai,
% Journal of Engineering Mechanics, Vol.  119, No.  3,  March, 1993.

function CB_elnasizz_nl
    %%
    % Parameters:
    E=20650;%MPa
    nu=0.3;
    b=0.76;%m
    h=1.22;%m
    P=123.8;
    n=3;% number of elements per member
    utol=1e-6;
    
    
    %%
    % Cross-sectional properties
    Parameters=Beam_cross_section(struct('type','rectangle','d2',h,'d3',b));
    
    tolerance=b/1000;% geometrical tolerance for merging nodes
    
    
    %%
    % Locations of the joints
    X=[0,0,6.1;
        -24.38,0,0;
        -24.38*cos(60/180*pi),-24.38*sin(60/180*pi),0;
        24.38*cos(60/180*pi),-24.38*sin(60/180*pi),0;
        24.38,0,0;
        24.38*cos(60/180*pi),24.38*sin(60/180*pi),0;
        -24.38*cos(60/180*pi),24.38*sin(60/180*pi),0;
        -12.57,0,4.55;
        -12.57*cos(60/180*pi),-12.57*sin(60/180*pi),4.55;
        12.57*cos(60/180*pi),-12.57*sin(60/180*pi),4.55;
        12.57,0,4.55;
        12.57*cos(60/180*pi),12.57*sin(60/180*pi),4.55;
        -12.57*cos(60/180*pi),12.57*sin(60/180*pi),4.55];
    members=[2,8;3,9;4,10;5,11;6,12;7,13;
        8,9;9,10;10,11;11,12;12,13;13,8;
        8,1;9,1;10,1;11,1;12,1;13,1
        ];
    % Generate the mesh member by member and merge together
    Parameters.x1x2_vector=[0 0 1];
    [fens,gcells] = Beam_member(X(members(1,:),:),n,@gcellset_beam3,Parameters);
    for i=2:size(members,1)
        [fens1,gcells1] = Beam_member(X(members(i,:),:),n,@gcellset_beam3,Parameters);
        [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tolerance);
        gcells= cat(gcells1,gcells2);
    end
    crownn=fenode_select(fens, struct('box', update_box([],[0 0 6.1]),'inflate', tolerance));
    
    %%
    % Material properties
    prop = property_linel_iso (struct('E',E,'nu',nu));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
    
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u0=0*geom0;%total displacement field
    % rotation field
    Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=fenode_select(fens, struct('box', [-inf inf -inf inf 0 0],'inflate', tolerance));
    ebc_prescribed=[1];
    ebc_comp=[];% clamped
    ebc_val=ebc_fenids*0;
    dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi  = apply_ebc (dchi);
    
    % Number equations
    dchi  = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    load_parameters=0.1:0.1:0.8;
    crown_displacement=[];
    for load_parameter=load_parameters
        
        % Initialization
        dchi = 0*dchi; % guess
        dchi = apply_ebc(dchi);
        u1 = u0;
        Rfield1=Rfield0;
        nl=nodal_load(struct ('id',crownn,'dir',[3],'magn',[-load_parameter*P]));
        
        iter=1;
        while 1
            feb1=feb;  % Make a copy of the state
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            K = start (sparse_sysmat, neqns);
            K = assemble (K, cat(2,stiffness(feb1,geom0,u1,Rfield1,dchi),stiffness_geo(feb1,geom0,u1,Rfield1,dchi)));
            dchi = scatter_sysvec(dchi, K\F); % Displacement increment
            % Line search
            R0 = dot(get(F,'vec'),gather_sysvec(dchi));
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,...
                u1+slice(dchi,(1:3),'transl'),update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat')),dchi);       % Internal forces
            F = assemble (F, evs);
            R1 = dot(get(F,'vec'),gather_sysvec(dchi));
            a = R0/R1;
            if ( a<0 )
                eta = a/2 +sqrt((a/2)^2 -a);
            else
                eta =a/2;
            end
            eta=min( [eta, 1.0] );
            u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
            Rfield1=update_rotation_field(Rfield1,eta*slice(dchi,(4:6),'rotat'));
            disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
        u0 = u1;                                               % update the displacement
        Rfield0=Rfield1; % update the rotations
        utip=gather(u1,get(nl, 'id'),'values');
        disp(['Crown deflection =' num2str(utip(3),'%27.15e\n')])
        crown_displacement=[crown_displacement,utip(3)];
        
        %%
        % Graphic display of the mesh and the deformed beam
        gv=graphic_viewer;
        gv=reset (gv,[]);
        scale=1;
        draw(feb, gv, struct ('x', geom0, 'u', [], 'R', [], 'facecolor','none'));
        draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
        %     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
        
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        %         draw(fens,gv, struct ('x', geom0, 'u', [], 'color','blue','offset',0.35)); 
        draw_axes(gv, struct([]));
        camset (gv, [-224.6266 -292.7391  216.4415         0         0    3.4053         0         0    1.0000    4.3101]);
        pause(1)
    end
    
    
    %%
    % Comparison with reference solution.
    izzelna_xy =[
        0         0
        0.2511    0.2048
        0.4266    0.3520
        0.5741    0.4449
        0.7878    0.5354
        1.0141    0.5933
        1.2075    0.6187
        1.3855    0.6344
        1.5966    0.6440
        1.8001    0.6488
        1.9731    0.6537
        2.1588    0.6645
        2.3521    0.6826
        2.4411    0.6923];
    shi_xy =[
        0         0
        0.2425    0.2026
        0.3495    0.2944
        0.4437    0.3693
        0.5634    0.4563
        0.7086    0.5276
        0.8715    0.5856
        1.0956    0.6436
        1.3019    0.6701
        1.5260    0.6834
        1.7170    0.6870
        1.9156    0.6931
        2.1474    0.7015
        2.3104    0.7124
        2.4402    0.7233];
    
    figure
    plot(izzelna_xy(:,1),izzelna_xy(:,2),'r-','linewidth',3)
    hold on
    plot(-crown_displacement,load_parameters,'x','linewidth',3)
    labels('Load parameter','Crown deflection [m]')
    legend({' lzzuddin, Elnashai',' Present'},'Location','Southeast')
    grid on
    set_graphics_defaults
end