% Find the nodes at a joint  between two beam members.
% 
% function [node1, node2]= Beam_join(fens,gcells, label1, label2, tolerance)
%
% Inputs:
% fens,gcells= Finite element nodes and geometric cells of the mesh, 
% label1, label2, 
% tolerance = Geometrical tolerance to use for joining nodes 
% 
% Outputs:
% node1, node2= serial numbers of nodes to be joined
%
% Example:
% % Cross-section parameters of member 1. Note that its  finite elements
% % (geometrical cells) are labeled 1. 
% Parameters=struct ('A',A,'I2',I,'I3',I,'J',2*I,...
%     'x1x2_vector',[0 0 1], 'label',1);
% % Mesh of member 1
% [fens1,gcells1] = Beam_member([[0 L 0];[L L 0]],n,@gcellset_beam3,Parameters);
% % Cross-section parameters of member 2. Note that its  finite elements
% % (geometrical cells) are labeled 2.
% Parameters=struct ('A',A,'I2',I,'I3',I,'J',2*I,...
%     'x1x2_vector',[0 0 1], 'label',2);
% % Mesh of member 2
% [fens2,gcells2] = Beam_member([[L L 0];[L 0 0]],n,@gcellset_beam3,Parameters);
% % Merge meshes, but make sure that no nodes are merged by using
% % the tolerance of 0.
% [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, 0);
% gcells= cat(gcells1,gcells2);
% % Set up the link between the nodes. In this case all the translations are
% % the same (spherical joint).
% link.component= [1,2,3]; % spherical joint
% % Now find which nodes are at the location where the beam members should be joined.
% [link.master, link.slave]= Beam_join(fens,gcells, 1, 2, L/1000);
% % ...
% % And finally , use the established joint link to number the equations.
%  ur  = numbereqns (ur, {link}); 
% 
function [node1, node2]= Beam_join(fens,gcells, label1, label2, tolerance)
    l1 =gcell_select(fens,gcells, struct('label',label1));
    m = fenode_to_gcell_map (struct('gcells',subset(gcells, l1)));
    gmap=get (m,'gcell_map');
    endn1=[];
    for i=1:length(gmap)
        if (length(gmap{i})==1)
            endn1= [endn1,i];
        end
    end 
    l2 =gcell_select(fens,gcells, struct('label',label2));
    m = fenode_to_gcell_map (struct('gcells',subset(gcells, l2)));
    gmap=get (m,'gcell_map');
    endn2=[];
    for i=1:length(gmap)
        if (length(gmap{i})==1)
            endn2= [endn2,i];
        end
    end 
    if (length(endn1)~=2)
        error('The beam 1 does not have two ends?');
    end
    if (length(endn2)~=2)
        error('The beam 2 does not have two ends?');
    end
    xyz=get(fens, 'xyz' );
    for i=[1,2]
        if (norm(xyz(endn1(1),:)-xyz(endn2(i),:))<= tolerance)
            node1=endn1(1); node2=endn2(i); 
            return
        end
        if (norm(xyz(endn1(2),:)-xyz(endn2(i),:))<= tolerance)
            node1=endn1(2); node2=endn2(i); 
            return
        end
    end 
    node1=[]; node2=[]; 
end